// =============================================================
// Extract Pollution Data to Senegal
// =============================================================
// Define Area of Dakar ========================================

var geometry = /* color: #d63000 */ee.Geometry.Polygon(
        [[[-17.444886623781144, 14.769712977427991],
          [-17.472352444093644, 14.765729195544859],
          [-17.480592190187394, 14.771040888495113],
          [-17.511491238038957, 14.763073300394572],
          [-17.53689712182802, 14.745809191356093],
          [-17.477845608156144, 14.698657922967907],
          [-17.475785671632707, 14.688695083728364],
          [-17.475099026124894, 14.68006025569746],
          [-17.45999282495302, 14.672089342607194],
          [-17.450379787843644, 14.666110967240408],
          [-17.44076675073427, 14.648839188204088],
          [-17.425660549562394, 14.647510533422905],
          [-17.41604751245302, 14.673417848289201],
          [-17.41879409448427, 14.6920160806151],
          [-17.417420803468644, 14.704635408409043],
          [-17.42428725854677, 14.707292016078764],
          [-17.428407131593644, 14.715261645078053],
          [-17.42428725854677, 14.721902780195242],
          [-17.411927639406144, 14.729871875352984],
          [-17.376222072999894, 14.737840679121089],
          [-17.361802517335832, 14.734520379638854],
          [-17.335709988038957, 14.724559177615596],
          [-17.36386245385927, 14.802908318157238]]]);

// Define Function =============================================
var extract_pollution = function(region, begin_date, end_date, agg_time, dataset, gdrive_folder, image_name_root){
  // agg_time: NOT PARAMETERIZED YET
  
  // List of dates =============================================
  // https://gis.stackexchange.com/questions/269313/mapping-over-list-of-dates-in-google-earth-engine
  var Date_Start = ee.Date(begin_date);
  var Date_End = ee.Date(end_date);
  var Date_window = ee.Number(1);
  
  // Create list of dates for time series
  var n_months = Date_End.difference(Date_Start,'hour').round();
  var dates = ee.List.sequence(0,n_months,1);
  var make_datelist = function(n) {
    return Date_Start.advance(n,'hour');
  };
  dates = dates.map(make_datelist);
  // Loop through dates; make image collection ================
  var fnc = function(d1) {
    var start = ee.Date(d1);
    var end = ee.Date(d1).advance(1,'hour');
    var date_range = ee.DateRange(start,end);
    
    // Define a Point object.
    var country = geometry;
    //var country = country.buffer({'distance': 35000});
    
    if(dataset == 'UV_aerosol'){
      var image = ee.ImageCollection('COPERNICUS/S5P/OFFL/L3_AER_AI'); 
      var image = image.select('absorbing_aerosol_index');
    };
    
    if(dataset == 'CO'){
      var image = ee.ImageCollection('COPERNICUS/S5P/OFFL/L3_CO'); 
      var image = image.select('CO_column_number_density');
    };
    
    if(dataset == 'HCHO'){
      var image = ee.ImageCollection('COPERNICUS/S5P/OFFL/L3_HCHO'); 
      var image = image.select('tropospheric_HCHO_column_number_density');
    };
    
    if(dataset == 'NO2'){
      var image = ee.ImageCollection('COPERNICUS/S5P/OFFL/L3_NO2'); 
      var image = image.select('tropospheric_NO2_column_number_density');
    };
    
    if(dataset == 'O3'){
      var image = ee.ImageCollection('COPERNICUS/S5P/OFFL/L3_O3'); 
      var image = image.select('O3_column_number_density');
    };
    
    if(dataset == 'SO2'){
      var image = ee.ImageCollection('COPERNICUS/S5P/OFFL/L3_SO2'); 
      var image = image.select('SO2_column_number_density');
    };
    
    if(dataset == 'CH4'){
      var image = ee.ImageCollection('COPERNICUS/S5P/OFFL/L3_CH4'); 
      var image = image.select('CH4_column_volume_mixing_ratio_dry_air');
    };
    
    if(dataset == 'AOD_047'){
      var image = ee.ImageCollection('MODIS/006/MCD19A2_GRANULES'); 
      var image = image.select('Optical_Depth_047');
    };
    
    if(dataset == 'AOD_055'){
      var image = ee.ImageCollection('MODIS/006/MCD19A2_GRANULES'); 
      var image = image.select('Optical_Depth_055');
    };
    //PM2.5 = DUSMASS25 + OCSMASS+ BCSMASS + SSSMASS25 + SO4SMASS* (132.14/96.06)
    if(dataset == 'BCSMASS'){
      var image = ee.ImageCollection("NASA/GSFC/MERRA/aer/2"); 
      var image = image.select('BCSMASS');
    };
  
    if(dataset == 'DUSMASS25'){
      var image = ee.ImageCollection("NASA/GSFC/MERRA/aer/2"); 
      var image = image.select('DUSMASS25');
    };
    
    if(dataset == 'OCSMASS'){
      var image = ee.ImageCollection("NASA/GSFC/MERRA/aer/2"); 
      var image = image.select('OCSMASS');
    };   
    if(dataset == 'SSSMASS25'){
      var image = ee.ImageCollection("NASA/GSFC/MERRA/aer/2"); 
      var image = image.select('SSSMASS25');
    }; 
    if(dataset == 'SO4SMASS'){
      var image = ee.ImageCollection("NASA/GSFC/MERRA/aer/2"); 
      var image = image.select('SO4SMASS');
    }; 
    image = ee.ImageCollection(
      image.filterDate(start,end)
    );
    
    image = image.median()
  
  // This function adds a band representing the image timestamp.
    var addTime = function(image) {
      return image.addBands(image.getNumber('system:time_start'));
    };
    
    // Map the function over the collection and display the result.
    //print(collection.map(addTime));
    
    // Make sure data types are all the same 
    //image = image.map(float())
    //image=addTime.map(image)
    image = image.float()
    image = image.clip(country)
      
    return(image);
  };
  
  var list_of_images = dates.map(fnc);
  var images_ic = ee.ImageCollection(list_of_images);
  
  // STACK BANDS ================================================
  
  // Function to turn imagecollection into image with bands
  var stackCollection = function(collection) {
    // Create an initial image.
    var first = ee.Image(collection.first()).select([]);
    //var time=first.addBands(first.getNumber('system:time_start'));
    
  
    // Write a function that appends a band to an image.
    var appendBands = function(image, previous) {
      return ee.Image(previous).addBands(image);
    };
    return ee.Image(collection.iterate(appendBands, first));
  };
  
  var images_stacked = stackCollection(images_ic);
  
  // Export ====================================================
  var image_name = image_name_root + '_' + dataset + '_' + begin_date + '_' + end_date;
  
  if(dataset == 'AOD_047' | dataset == 'AOD_055'){
    var resolution = 1000;
  } else{
    var resolution = 1113.2;
  }
  
  Export.image.toDrive({
    folder: gdrive_folder,
    image: images_stacked,
    scale: resolution,
    region: region,
    description: image_name,
  });
  
  return images_stacked;
  
};

// Implement Function ============================================
var region = geometry;
//var region = region.buffer({'distance': 70000});

//  MERRA-2
var out = extract_pollution(region, '2021-01-01', '2021-12-31', 'daily', 'DUSMASS25', 'RemoteSensing', 'dakar');
var out = extract_pollution(region, '2021-01-01', '2021-12-31', 'daily', 'OCSMASS', 'RemoteSensing', 'dakar');
var out = extract_pollution(region, '2021-01-01', '2021-12-31', 'daily', 'BCSMASS', 'RemoteSensing', 'dakar');
var out = extract_pollution(region, '2021-01-01', '2021-12-31', 'daily', 'SSSMASS25', 'RemoteSensing', 'dakar');
var out = extract_pollution(region, '2021-01-01', '2021-12-31', 'daily', 'SO4SMASS', 'RemoteSensing', 'dakar');

// MODIS
var out = extract_pollution(region, '2019-01-01', '2019-12-31', 'daily', 'AOD_047', 'RemoteSensing', 'dakar');
var out = extract_pollution(region, '2019-01-01', '2019-12-31', 'daily', 'AOD_055', 'RemoteSensing', 'dakar');

// Plot Region
var out = extract_pollution(region, '2019-01-01', '2019-12-31', 'daily', 'NO2', 'RemoteSensing', 'dakar');
var out = extract_pollution(region, '2019-01-01', '2019-12-31', 'daily', 'O3', 'RemoteSensing', 'dakar');


Map.addLayer(out, {}, 'Pollution');



