// =============================================================
// Extract Pollution Data to Senegal
// =============================================================

var geometry = /* color: #d63000 */ee.Geometry.Polygon(
        [[[-17.444886623781144, 14.769712977427991],
          [-17.472352444093644, 14.765729195544859],
          [-17.480592190187394, 14.771040888495113],
          [-17.511491238038957, 14.763073300394572],
          [-17.53689712182802, 14.745809191356093],
          [-17.477845608156144, 14.698657922967907],
          [-17.475785671632707, 14.688695083728364],
          [-17.475099026124894, 14.68006025569746],
          [-17.45999282495302, 14.672089342607194],
          [-17.450379787843644, 14.666110967240408],
          [-17.44076675073427, 14.648839188204088],
          [-17.425660549562394, 14.647510533422905],
          [-17.41604751245302, 14.673417848289201],
          [-17.41879409448427, 14.6920160806151],
          [-17.417420803468644, 14.704635408409043],
          [-17.42428725854677, 14.707292016078764],
          [-17.428407131593644, 14.715261645078053],
          [-17.42428725854677, 14.721902780195242],
          [-17.411927639406144, 14.729871875352984],
          [-17.376222072999894, 14.737840679121089],
          [-17.361802517335832, 14.734520379638854],
          [-17.335709988038957, 14.724559177615596],
          [-17.36386245385927, 14.802908318157238]]]);

// ============================================================= 
// Extract Pollution Data to Senegal
// =============================================================

// Define Function =============================================
var extract_pollution = function(region, begin_date, end_date, dataset, gdrive_folder, image_name_root){
  
  // List of dates =============================================
  var Date_Start = ee.Date(begin_date);
  var Date_End = ee.Date(end_date);
  var Date_window = ee.Number(1);
  
  // Create list of dates for time series
  var n_months = Date_End.difference(Date_Start,'hour').round();
  var dates = ee.List.sequence(0,n_months,1);
  var make_datelist = function(n) {
    return Date_Start.advance(n,'hour');
  };
  dates = dates.map(make_datelist);

  // Function to process each date
  var fnc = function(d1) {
    var start = ee.Date(d1);
    var end = ee.Date(d1).advance(1,'hour');
    var country = geometry;
    
    // MODIS datasets
    var image;
    if (dataset == 'AOD_047') {
      image = ee.ImageCollection('MODIS/006/MCD19A2_GRANULES')
        .select('Optical_Depth_047');
    }
    
    if (dataset == 'AOD_055') {
      image = ee.ImageCollection('MODIS/006/MCD19A2_GRANULES')
        .select('Optical_Depth_055');
    }

    // Filter and get median
    image = ee.ImageCollection(image.filterDate(start, end)).median();

    // Convert to float, clip to region
    image = image.float().clip(country);

    return image;
  };
  
  // Map the function over the list of dates
  var list_of_images = dates.map(fnc);
  var images_ic = ee.ImageCollection(list_of_images);

  // Function to stack the images into a single multi-band image
  var stackCollection = function(collection) {
    var first = ee.Image(collection.first()).select([]);
    var appendBands = function(image, previous) {
      return ee.Image(previous).addBands(image);
    };
    return ee.Image(collection.iterate(appendBands, first));
  };
  
  // Stack images
  var images_stacked = stackCollection(images_ic);
  
  // Export ====================================================
  var image_name = image_name_root + '_' + dataset + '_' + begin_date + '_' + end_date;
  
  var resolution = 1000; // MODIS resolution
  
  // Print dataset name
  print('Downloading: ', image_name);

  // Export to Google Drive
  Export.image.toDrive({
    folder: gdrive_folder,
    image: images_stacked,
    scale: resolution,
    region: region,
    description: image_name,
  });

  return images_stacked;
};

// Implement Function ============================================
var region = geometry;

// Download AOD_047
var out_AOD_047 = extract_pollution(region, '2001-01-01', '2022-06-15', 'AOD_047', 'RemoteSensing', 'dakar');

// Download AOD_055
var out_AOD_055 = extract_pollution(region, '2001-01-01', '2022-06-15', 'AOD_055', 'RemoteSensing', 'dakar');

// Display on map
Map.addLayer(out_AOD_047, {}, 'AOD_047');
Map.addLayer(out_AOD_055, {}, 'AOD_055');
