// =============================================================
// Extract Pollution Data to Senegal
// =============================================================

var geometry = /* color: #d63000 */ee.Geometry.Polygon(
        [[[-17.444886623781144, 14.769712977427991],
          [-17.472352444093644, 14.765729195544859],
          [-17.480592190187394, 14.771040888495113],
          [-17.511491238038957, 14.763073300394572],
          [-17.53689712182802, 14.745809191356093],
          [-17.477845608156144, 14.698657922967907],
          [-17.475785671632707, 14.688695083728364],
          [-17.475099026124894, 14.68006025569746],
          [-17.45999282495302, 14.672089342607194],
          [-17.450379787843644, 14.666110967240408],
          [-17.44076675073427, 14.648839188204088],
          [-17.425660549562394, 14.647510533422905],
          [-17.41604751245302, 14.673417848289201],
          [-17.41879409448427, 14.6920160806151],
          [-17.417420803468644, 14.704635408409043],
          [-17.42428725854677, 14.707292016078764],
          [-17.428407131593644, 14.715261645078053],
          [-17.42428725854677, 14.721902780195242],
          [-17.411927639406144, 14.729871875352984],
          [-17.376222072999894, 14.737840679121089],
          [-17.361802517335832, 14.734520379638854],
          [-17.335709988038957, 14.724559177615596],
          [-17.36386245385927, 14.802908318157238]]]);


// Define Function =============================================
var extract_pollution = function(region, begin_date, end_date, agg_time, dataset, gdrive_folder, image_name_root){
  var Date_Start = ee.Date(begin_date);
  var Date_End = ee.Date(end_date);
  var n_months = Date_End.difference(Date_Start, 'hour').round();
  var dates = ee.List.sequence(0, n_months, 1);
  var make_datelist = function(n) {
    return Date_Start.advance(n, 'hour');
  };
  dates = dates.map(make_datelist);

  // Function to process a single date range
  var fnc = function(d1) {
    var start = ee.Date(d1);
    var end = ee.Date(d1).advance(1, 'hour');
    var country = geometry;
    var image;

    // Dataset selection
    if(dataset == 'UV_aerosol'){
      image = ee.ImageCollection('COPERNICUS/S5P/OFFL/L3_AER_AI').select('absorbing_aerosol_index');
    } else if(dataset == 'CO'){
      image = ee.ImageCollection('COPERNICUS/S5P/OFFL/L3_CO').select('CO_column_number_density');
    } else if(dataset == 'HCHO'){
      image = ee.ImageCollection('COPERNICUS/S5P/OFFL/L3_HCHO').select('tropospheric_HCHO_column_number_density');
    } else if(dataset == 'NO2'){
      image = ee.ImageCollection('COPERNICUS/S5P/OFFL/L3_NO2').select('tropospheric_NO2_column_number_density');
    } else if(dataset == 'O3'){
      image = ee.ImageCollection('COPERNICUS/S5P/OFFL/L3_O3').select('O3_column_number_density');
    } else if(dataset == 'SO2'){
      image = ee.ImageCollection('COPERNICUS/S5P/OFFL/L3_SO2').select('SO2_column_number_density');
    } else if(dataset == 'CH4'){
      image = ee.ImageCollection('COPERNICUS/S5P/OFFL/L3_CH4').select('CH4_column_volume_mixing_ratio_dry_air');
    } else if(dataset == 'AOD_047'){
      image = ee.ImageCollection('MODIS/006/MCD19A2_GRANULES').select('Optical_Depth_047');
    } else if(dataset == 'AOD_055'){
      image = ee.ImageCollection('MODIS/006/MCD19A2_GRANULES').select('Optical_Depth_055');
    } else if(dataset == 'BCSMASS'){
      image = ee.ImageCollection("NASA/GSFC/MERRA/aer/2").select('BCSMASS');
    } else if(dataset == 'DUSMASS25'){
      image = ee.ImageCollection("NASA/GSFC/MERRA/aer/2").select('DUSMASS25');
    } else if(dataset == 'OCSMASS'){
      image = ee.ImageCollection("NASA/GSFC/MERRA/aer/2").select('OCSMASS');
    } else if(dataset == 'SSSMASS25'){
      image = ee.ImageCollection("NASA/GSFC/MERRA/aer/2").select('SSSMASS25');
    } else if(dataset == 'SO4SMASS'){
      image = ee.ImageCollection("NASA/GSFC/MERRA/aer/2").select('SO4SMASS');
    }

    image = ee.ImageCollection(image.filterDate(start, end));
    image = image.median().float().clip(country);
    
    return image;
  };

  var list_of_images = dates.map(fnc);
  var images_ic = ee.ImageCollection(list_of_images);

  // STACK BANDS ================================================
  var stackCollection = function(collection) {
    var first = ee.Image(collection.first()).select([]);
    var appendBands = function(image, previous) {
      return ee.Image(previous).addBands(image);
    };
    return ee.Image(collection.iterate(appendBands, first));
  };
  
  var images_stacked = stackCollection(images_ic);
  
  // Export ====================================================
  var image_name = image_name_root + '_' + dataset + '_' + begin_date + '_' + end_date;
  var resolution = (dataset == 'AOD_047' || dataset == 'AOD_055') ? 1000 : 1113.2;
  
  Export.image.toDrive({
    folder: gdrive_folder,
    image: images_stacked,
    scale: resolution,
    region: region,
    description: image_name,
  });
  
  return images_stacked;
};

// Implement Function ============================================

// List of datasets
var datasets = ['BCSMASS', 'DUSMASS25', 'OCSMASS', 'SO4SMASS', 'SSSMASS25'];

// List of years
var years = [
  '2012', '2013', '2014', '2015', '2016', '2017', '2018', 
  '2019', '2020', '2021', '2022', '2023'
];
// Loop over datasets and years
datasets.forEach(function(dataset) {
  years.forEach(function(year) {
    var start_date = year + '-01-01';
    var end_date = (year == '2023') ? '2023-12-31' : year + '-12-31';

    // Print the name of the dataset being processed
    var dataset_name = 'Processing: ' + dataset + ' for year ' + year;
    print(dataset_name);  // This will print the dataset and year being processed

    extract_pollution(geometry, start_date, end_date, 'daily', dataset, 'RemoteSensing', 'dakar');
  });
});
