**************
**Calibration
**************


***********************************************
*Merging datasets to form calibration dataset
***********************************************

import delimited "$final_data_dir/full_local.csv", clear

gen clean_utcdatetime = trim(day_hr)
gen clean_datetime = regexs(1) if regexm(clean_utcdatetime, "([0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})")
gen double datetime = clock(clean_datetime, "YMD hms")
format datetime %tc


save "$final_data_dir/full_local", replace

import delimited "$raw_data_dir\cgqa_hourly.csv", clear
gen clean_utcdatetime = trim(day_hr)
gen clean_datetime = regexs(1) if regexm(clean_utcdatetime, "([0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})")
gen double datetime = clock(clean_datetime, "YMD hms")
format datetime %tc
extrdate year year = datetime 

*drop if year < 2020
merge 1:m datetime using "$final_data_dir/full_local"
tab year _merge
keep if _merge ==3

label variable humidity "Humidity"
label variable temp "Temperature"

save "$final_data_dir/calibration", replace


*********************
*Bel-air and FC002 calibration
*********************
*cf=1 and um

use "$final_data_dir/calibration", clear

gen pm25_cf1_fc002 = . 
replace pm25_cf1_fc002 = avg_pm25_cf1 if locationcode == "DAQ-F-C-002"
gen pm25_um_fc002 = . 
replace pm25_um_fc002 = avg_pm25_um if locationcode == "DAQ-F-C-002"

gen cgqa_calibrate_belair = pm25_belair_µgm
label variable cgqa_calibrate_belair "Bel-Air"

gen pa_cf1 = pm25_cf1_fc002
label variable pa_cf1 "PM2.5 (cf=1)"
gen pa_um = pm25_um_fc002
label variable pa_um "PM2.5 (\mu m)"


eststo: lasso linear cgqa_calibrate_belair pa_cf1  humidity   temp , fold(5)  rseed(87392798)
lassocoef, display(coef, post)

*Out of Sample R-Square: 0.6151

eststo: reg cgqa_calibrate_belair pa_cf1  humidity temp 

estadd scalar devratio_max = 0.6151, replace
eststo bel_air_cf1

// estadd scalar devratio_max = e(devratio_max), replace

esttab bel_air_cf1, scalars(devratio_max "Out of Sample R-Squre") label  nostar not noobs





lasso linear cgqa_calibrate_belair pa_um humidity  temp , fold(5) rseed(87392798)


*Out of Sample R-Square: 0.7990

lassocoef, display(coef, post)

eststo: reg cgqa_calibrate_belair pa_um humidity  temp 
estadd scalar devratio_max = 0.7990, replace

eststo bel_air_um


// estadd scalar devratio_max = e(devratio_max), replace

esttab bel_air_cf1 bel_air_um, scalars(devratio_max "Out of Sample R²") label


******************
*Pikine and FS011
******************

use "$final_data_dir/calibration", clear


gen pm25_cf1_fs011 = . 
replace pm25_cf1_fs011 = avg_pm25_cf1 if locationcode == "DAQ-F-S-011"
gen pm25_um_fs011 = . 
replace pm25_um_fs011 = avg_pm25_um if locationcode == "DAQ-F-S-011"

gen cgqa_calibrate_belair = pm25_belair_µgm
label variable cgqa_calibrate_belair "Bel-Air"
gen cgqa_calibrate_pikine = pm25_pikine_µgm
label variable cgqa_calibrate_pikine "Pikine"
gen pa_cf1 = pm25_cf1_fs011
label variable pa_cf1 "PM2.5 (cf=1)"
gen pa_um = pm25_um_fs011
label variable pa_um "PM2.5 (\mu m)"

eststo: lasso linear cgqa_calibrate_pikine pa_cf1  humidity  temp , fold(5) rseed(87392798)
*Out of Sample R-Square: 0.3650

lassocoef, display(coef, post)

eststo: reg cgqa_calibrate_pikine pa_cf1  humidity  temp 
estadd scalar devratio_max = 0.3650, replace

eststo pikine_cf1


esttab pikine_cf1, scalars(devratio_max "Out of Sample R²") label

*um
lasso linear cgqa_calibrate_pikine pa_um humidity  temp , fold(5) rseed(87392798)

*Out of Sample R-Square: 0.5147

lassocoef, display(coef, post)

eststo: reg cgqa_calibrate_pikine pa_um humidity  temp
estadd scalar devratio_max = 0.5147, replace

eststo pikine_um


esttab bel_air_cf1 bel_air_um pikine_cf1 pikine_um, scalars(devratio_max "Out of Sample R²") label 

***********************
*Both Belair and pikine
***********************
use "$final_data_dir/calibration", clear


gen pm25_cf1_fc002_fs011 = .
replace pm25_cf1_fc002_fs011 = avg_pm25_cf1 if locationcode == "DAQ-F-C-002" & year < 2023
replace pm25_cf1_fc002_fs011 = avg_pm25_cf1 if locationcode == "DAQ-F-S-011" &  year ==2023

gen pm25_um_fc002_fs011 = .
replace pm25_um_fc002_fs011 = avg_pm25_um if locationcode == "DAQ-F-C-002" & year < 2023
replace pm25_um_fc002_fs011 = avg_pm25_um if locationcode == "DAQ-F-S-011" &  year ==2023

gen pm25_belair_pikine = .
replace pm25_belair_pikine = pm25_belair_µgm if  year < 2023

replace pm25_belair_pikine = pm25_pikine_µgm if  year >= 2022 & year < = 2024



gen cgqa_calibrate_belair = pm25_belair_µgm
label variable cgqa_calibrate_belair "Bel-Air"
gen cgqa_calibrate_pikine = pm25_pikine_µgm
label variable cgqa_calibrate_pikine "Pikine"


gen cgqa_calibrate_both = pm25_belair_pikine
label variable cgqa_calibrate_both "Both Bel-Air and Pikine"
gen pa_cf1 = pm25_cf1_fc002_fs011
label variable pa_cf1 "PM2.5 (cf=1)"
gen pa_um = pm25_um_fc002_fs011
label variable pa_um "PM2.5 (\mu m)"




eststo: lasso linear cgqa_calibrate_both pa_cf1  humidity  temp , fold(5)  rseed(87392798)
*Out of Sample R-Square:  0.4491

lassocoef, display(coef, post)

eststo: reg cgqa_calibrate_both pa_cf1  humidity  temp 
estadd scalar devratio_max =  0.4491, replace


eststo both_cf1


esttab both_cf1, scalars(devratio_max "Out of Sample R²") label


lasso linear cgqa_calibrate_both pa_um humidity  temp , fold(5) rseed(87392798)
*Out of Sample R-Square: 0.5646

lassocoef, display(coef, post)

reg cgqa_calibrate_both pa_um humidity  temp 
estadd scalar devratio_max =  0.5646, replace

eststo both_um


	
esttab bel_air_cf1 bel_air_um pikine_cf1 pikine_um both_cf1 both_um using "${outputs}/calibrations_hourly.tex", type  ///
	scalars("devratio_max Out-of-Sample \[R^2\]") label replace nonotes noobs nonumbers  nostar not gaps   ///
	title("Calibration Coefficients comparing Regulatory and nearby Purple Air monitors (Hourly) \label{calibrations-hourly}") ///
	mtitles("CF=1" "\shortstack{\underline{Monitor at Bel-Air}\\ \mu g/m^3}" "CF=1" "\shortstack{\underline{Monitor at Pikine}\\ \mu g/m^3}" "CF=1" "\shortstack{\underline{Bel-air \& Pikine}\\ \mu g/m^3}") ///
	 coeflabels(pa_cf1 "PM2.5 (cf=1)" pa_um "PM2.5 (\mu m)"  humidity "Humidity" temp "Temperature") ///
	addnotes("\textbf{Note}: Lasso linear model with cross-validation fold(5). Using the Purple Air cf=1 formula and particulate matter concentration (µm)." ) ///
	order(pa_cf1 pa_um)

*************************************
*Daily Calibration
*************************************

******************
*Generate variables
*******************

use "$final_data_dir/calibration", clear


gen pm25_cf1_fc002 = . 
replace pm25_cf1_fc002 = avg_pm25_cf1 if locationcode == "DAQ-F-C-002"
gen pm25_um_fc002 = . 
replace pm25_um_fc002 = avg_pm25_um if locationcode == "DAQ-F-C-002"


gen pm25_cf1_fs011 = . 
replace pm25_cf1_fs011 = avg_pm25_cf1 if locationcode == "DAQ-F-S-011"
gen pm25_um_fs011 = . 
replace pm25_um_fs011 = avg_pm25_um if locationcode == "DAQ-F-S-011"


gen pm25_cf1_fc002_fs011 = .
replace pm25_cf1_fc002_fs011 = avg_pm25_cf1 if locationcode == "DAQ-F-C-002" & year < 2023
replace pm25_cf1_fc002_fs011 = avg_pm25_cf1 if locationcode == "DAQ-F-S-011" &  year ==2023

gen pm25_um_fc002_fs011 = .
replace pm25_um_fc002_fs011 = avg_pm25_um if locationcode == "DAQ-F-C-002" & year < 2023
replace pm25_um_fc002_fs011 = avg_pm25_um if locationcode == "DAQ-F-S-011" &  year ==2023

gen pm25_belair_pikine = .
replace pm25_belair_pikine = pm25_belair_µgm if  year < 2023
replace pm25_belair_pikine = pm25_pikine_µgm if  year >= 2022 & year < = 2024

************
*Collapse to daily dataset
************

* Extract year and month
extrdate month month = datetime 
extrdate week week = datetime 
extrdate day day = datetime 
extrdate dow dow = datetime 
generate hour = hh(datetime)
gen date = mdy(month, day, year)
format date   %td 

collapse (mean) pm25_belair_pikine pm25_belair_µgm pm25_pikine_µgm temp precip humidity   pm25_cf1_fc002 pm25_um_fc002 pm25_cf1_fs011 pm25_um_fs011  pm25_cf1_fc002_fs011 pm25_um_fc002_fs011, by(date)


label variable humidity "Humidity"
label variable temp "Temperature"


save "$final_data_dir/calibration_daily", replace


********************
*Bel-air and FC002
********************
gen cgqa_calibrate_belair = pm25_belair_µgm
label variable cgqa_calibrate_belair "Bel-Air"
gen pa_cf1 = pm25_cf1_fc002
label variable pa_cf1 "PM2.5 (cf=1)"
gen pa_um = pm25_um_fc002
label variable pa_um "PM2.5 (\mu m)"


eststo: lasso linear cgqa_calibrate_belair pa_cf1  humidity   temp , fold(5)  rseed(87392798)
*Out of Sample R-Square:  0.6961

lassocoef, display(coef, post)
eststo: reg cgqa_calibrate_belair pa_cf1  humidity   temp 
estadd scalar devratio_max =  0.6961, replace

eststo bel_air_cf1
esttab bel_air_cf1, scalars(devratio_max "Out of Sample R²") label replace

eststo: lasso linear cgqa_calibrate_belair pa_um humidity  temp , fold(5)  rseed(87392798)
*Out of Sample R-Square:  0.9093 

lassocoef, display(coef, post)
eststo: reg cgqa_calibrate_belair pa_um humidity  temp 
estadd scalar devratio_max =  0.9093, replace

eststo bel_air_um
esttab bel_air_cf1 bel_air_um, scalars(devratio_max "Out of Sample R²") label replace


******************
*Pikine and FS011
******************
use "$final_data_dir/calibration_daily", clear


gen cgqa_calibrate_belair = pm25_belair_µgm
label variable cgqa_calibrate_belair "Bel-Air"
gen cgqa_calibrate_pikine = pm25_pikine_µgm
label variable cgqa_calibrate_pikine "Pikine"
gen pa_cf1 = pm25_cf1_fs011
label variable pa_cf1 "PM2.5 (cf=1)"
gen pa_um = pm25_um_fs011
label variable pa_um "PM2.5 (\mu m)"


eststo: lasso linear cgqa_calibrate_pikine pa_cf1  humidity  temp , fold(5) rseed(87392798)


*Out of Sample R-Square: 0.6922

lassocoef, display(coef, post)
eststo pikine_cf1
eststo: reg cgqa_calibrate_pikine pa_cf1  humidity  temp 
estadd scalar devratio_max =  0.6922, replace

esttab pikine_cf1, scalars(devratio_max "Out of Sample R²") label


lasso linear cgqa_calibrate_pikine pa_um humidity  temp , fold(5) rseed(87392798) 
*Out of Sample R-Square:   0.7628

lassocoef, display(coef, post)
reg cgqa_calibrate_pikine pa_um humidity  temp 
estadd scalar devratio_max =   0.7628, replace

eststo pikine_um
esttab bel_air_cf1 bel_air_um pikine_cf1 pikine_um, scalars(devratio_max "Out of Sample R²") label 

***********************
*Both Belair and pikine
***********************
use "$final_data_dir/calibration_daily", clear



gen cgqa_calibrate_belair = pm25_belair_µgm
label variable cgqa_calibrate_belair "Bel-Air"
gen cgqa_calibrate_pikine = pm25_pikine_µgm
label variable cgqa_calibrate_pikine "Pikine"


gen cgqa_calibrate_both = pm25_belair_pikine
label variable cgqa_calibrate_both "Both Bel-Air and Pikine"
gen pa_cf1 = pm25_cf1_fc002_fs011
label variable pa_cf1 "PM2.5 (cf=1)"
gen pa_um = pm25_um_fc002_fs011
label variable pa_um "PM2.5 (\mu m)"


eststo: lasso linear cgqa_calibrate_both pa_cf1  humidity  temp , fold(5)  rseed(87392798) 

*Out of Sample R-Square:   0.7194


eststo: reg cgqa_calibrate_both pa_cf1  humidity  temp
estadd scalar devratio_max =   0.7194, replace
  
eststo both_cf1

esttab both_cf1, scalars(devratio_max "Out of Sample R²") label


eststo:lasso linear cgqa_calibrate_both pa_um humidity  temp , fold(5)  rseed(87392798) 
eststo:reg cgqa_calibrate_both pa_um humidity  temp 
estadd scalar devratio_max =   0.7800, replace
eststo both_um

*Out of Sample R-Square:    0.7800

esttab both_um, scalars(devratio_max "Out of Sample R²") label


	
	
esttab bel_air_cf1 bel_air_um pikine_cf1 pikine_um both_cf1 both_um using "${outputs}/calibrations_daily.tex", type ///
	scalars("devratio_max Out-of-Sample \[R^2\]") label replace nonotes noobs nonumbers  nostar not gaps ///
	title("Calibration Coefficients comparing Regulatory and nearby Purple Air monitors (Daily) \label{calibrations-daily}") ///
	mtitles("CF=1" "\shortstack{\underline{Monitor at Bel-Air}\\ \mu g/m^3}" "CF=1" "\shortstack{\underline{Monitor at Pikine}\\ \mu g/m^3}" "CF=1" "\shortstack{\underline{Bel-air \& Pikine}\\ \mu g/m^3}") ///
	 coeflabels(pa_cf1 "PM2.5 (cf=1)" pa_um "PM2.5 (\mu m)"  humidity "Humidity" temp "Temperature") ///
	addnotes("Lasso linear model with cross-validation fold(5)" "Using the Purple Air cf=1 formula and particulate matter concentration (µm)" ) ///
	order(pa_cf1 pa_um)

	
esttab bel_air_cf1 bel_air_um pikine_cf1 pikine_um both_cf1 both_um using "${outputs}/calibrations_daily.tex", type ///
	scalars("devratio_max Out-of-Sample \[R^2\]") label replace nonotes noobs nonumbers  nostar not gaps ///
	title("Calibration Coefficients comparing Regulatory and nearby Purple Air monitors (Daily) \label{calibrations-daily}") ///
	mtitles("CF=1" "\shortstack{\underline{Monitor at Bel-Air}\\ \mu g/m^3}" "CF=1" "\shortstack{\underline{Monitor at Pikine}\\ \mu g/m^3}" "CF=1" "\shortstack{\underline{Bel-air \& Pikine}\\ \mu g/m^3}") ///
	 coeflabels(pa_cf1 "PM2.5 (cf=1)" pa_um "PM2.5 (\mu m)"  humidity "Humidity" temp "Temperature") ///
	addnotes("\textbf{Note}: Lasso linear model with cross-validation fold(5). Using the Purple Air cf=1 formula and particulate matter concentration (µm)" ) ///
	order(pa_cf1 pa_um)	
// **************************************************************************
// *Calibrate against satellite data
// **************************************************************************
// import delimited "$raw_data_dir/covid_merra.csv", clear
// rename pm25 pm25_merra
// save "$final_data_dir/covid_merra", replace
//
//
// ****************
// *Merge for PA and CGQA data
// *****************
// import delimited "$raw_data_dir/PA_CGQA_20240401.csv", clear 
// merge 1:1 day_hr using "$raw_data_dir/hourly_humidity"
// drop if _merge==1
// drop _merge
//
//
// merge 1:1 day_hr using "$raw_data_dir/covid_merra"
// keep if _merge==3
//
// *gen hour=clock(day_hr, "YMDhms")
// format hour %tc
//
// gen day=substr(day_hr,1 ,10)
//
// *Test against average of all monitors
// *cf1
//
// lasso linear pm25_merra humidity pm2_5_cf_1_avg temp , fold(5) rseed(87392798)
// *Out of Sample R-Square:   0.3509
// eststo: reg pm25_merra humidity pm2_5_cf_1_avg temp 
// estadd scalar devratio_max =   0.3509, replace
//
// eststo satellite_cf1_hourly
// // estadd scalar devratio_max = e(devratio_max), replace
// esttab satellite_cf1_hourly, scalars(devratio_max "Out of Sample R²") label
//
//
// *um
// lasso linear pm25_merra humidity pm2_5_um_avg temp , fold(5)  rseed(87392798)
// *Out of Sample R-Square:   0.3775
//
// eststo: reg pm25_merra humidity pm2_5_um_avg temp 
// estadd scalar devratio_max =   0.3775, replace
//
// eststo satellite_um_hourly
// esttab satellite_um_hourly, scalars(devratio_max "Out of Sample R²") label
//
//
//
//
//
// *********************
// *Daily
// ********************
//
// collapse (mean) humidity temp pm25_merra pm2_5_cf_1_avg pm2_5_um_*  (max) pm25_merra_max=pm25_merra , by(date)
//
// *cf1
// lasso linear pm25_merra humidity pm2_5_cf_1_avg temp , fold(5)  rseed(87392798)
// *Out of Sample R-Square:   0.5344
//
// eststo: reg pm25_merra humidity pm2_5_cf_1_avg temp  
// estadd scalar devratio_max =   0.5344, replace
//
// eststo satellite_cf1_daily
// esttab satellite_cf1_daily, scalars(devratio_max "Out of Sample R²") label
//
// lasso linear pm25_merra humidity pm2_5_um_avg temp , fold(5)  rseed(87392798)
// *Out of Sample R-Square:   0.5706
//
// eststo: reg pm25_merra humidity pm2_5_um_avg temp
// estadd scalar devratio_max =   0.5706, replace
//
// eststo satellite_um_daily
// esttab satellite_um_daily, scalars(devratio_max "Out of Sample R²") label
//
//
//
//				
// *updated table notes 
// esttab satellite_cf1_hourly satellite_um_hourly satellite_cf1_daily satellite_um_daily using "${outputs}/calibrations_satellite.tex",  ///
//     prehead("\begin{table}[htbp]\centering \\ \caption{Calibration Coefficients comparing Satellite Measures and Purple Air monitors \label{calibrations-satellite}} \\ \begin{tabular}{l*{5}{c}} \toprule  ") ///
// 	 coeflabels(pm2_5_cf_1_avg "PM2.5 (cf=1)" pm2_5_um_avg "PM2.5 (\mu m)"  humidity "Humidity" temp "Temperature") ///
// 	mtitles("CF=1" "\shortstack{\underline{   Hourly   }  \\ \mu g/m^3}" "CF=1" "\shortstack{\underline{   Daily   }  \\ \mu g/m^3}") ///
// 	scalars("devratio_max Out-of-Sample \[ R^2 \]") label replace nonotes noobs nonumbers  nostar not gaps ///
// postfoot("\bottomrule \multicolumn{5}{p{10cm}}{\footnotesize \textbf{Note}: Lasso linear model with cross-validation fold(5). Using the Purple Air cf=1 formula and particulate matter concentration (µm). Using an average of all Purple Air monitors across Dakar} \\ \end{tabular} \\\end{table}") ///
//     fragment		
//		
//		
******************************************
* END OF SCRIPT
******************************************	
