**************
**Correlations
**************


****************
*Merging dfs
****************
*
import delimited "$final_data_dir/full_local.csv", clear

gen clean_utcdatetime = trim(day_hr)
gen clean_datetime = regexs(1) if regexm(clean_utcdatetime, "([0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})")
gen double datetime = clock(clean_datetime, "YMD hms")
format datetime %tc

extrdate month month = datetime 
extrdate day day = datetime 
extrdate week week = datetime 
extrdate year year = datetime 

gen date = mdy(month, day, year)
format date %tdMM/DD/CCYY


bysort locationcode date: gen location_daily_obs = _N
replace avg_pm25_cf1 =. if location_daily_obs <=12

*Collapsing hourly acros all location_codes
collapse (mean)   avg_pm25_cf1 barkjohn jaffe_new local_hourly local_daily  satellite_hourly satellite_daily , by(datetime) 

save "$final_data_dir/full_local_hourly_averages", replace

*merge with cgqa
// import delimited "$Niall_dir\DIME-Senegal\datawork\Air_Quality\DataSets\Secondary\CGQA\processed\cgqa_hourly_05_15_2024.csv", clear

import delimited "$raw_data_dir/cgqa_hourly.csv", clear
gen clean_utcdatetime = trim(day_hr)
gen clean_datetime = regexs(1) if regexm(clean_utcdatetime, "([0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2})")
gen double datetime = clock(clean_datetime, "YMD hms")
format datetime %tc
extrdate year year = datetime 

keep pm25_reliable_µgm pm25_dakar_µgm datetime year


drop if year < 2020
merge 1:m datetime using "$final_data_dir/full_local_hourly_averages"
tab year _merge
drop _merge

save "$final_data_dir/cgqa_local", replace

*merge with merra-2



import delimited "$final_data_dir/covid_merra.csv", clear


gen clean_utcdatetime = trim(day_hr)
gen double datetime = clock(clean_utcdatetime, "YMD hms")

format datetime %tc

keep datetime pm25
extrdate year year = datetime 

drop if year < 2020 | year == 2024

merge 1:1 datetime using "$final_data_dir/cgqa_local"

tab year _merge
extrdate month month = datetime 
extrdate day day = datetime 
extrdate week week = datetime 
drop year
extrdate year year = datetime 
extrdate dow dow = datetime 


label variable avg_pm25_cf1 "Purple Air"
label variable jaffe_new "Jaffe"
label variable local_hourly "Local"
label variable pm25_reliable_µgm "CGQA"

*Drop values after May 12th for any year 
gen invalid_dates = (month > 5) | (month == 5 & day > 12)
*replace invalid_dates = 1 if year == 2021
replace invalid_dates = 1 if (month ==1 ) & (day <= 22)
tab year
// drop if invalid_dates == 1
// keep if year == 2020 | year ==2023 
keep if year == 2023

save "$final_data_dir/correlations", replace


* ****************************
*HOURLY
***************************
clear all
use  "$final_data_dir/correlations", clear


// drop if dow == 4 |dow == 0 |  dow == 5| dow == 6
// keep if dow == 0  | dow == 5| dow == 6


drop satellite_daily
rename satellite_hourly satellite_daily // so satellite coefficients are on the same line in the final table 

drop local_daily
rename local_hourly local_daily // so local coefficients are on the same line in the final table 



estpost  corr  pm25_reliable_µgm pm25 avg_pm25_cf1  jaffe_new local_daily satellite_daily 
eststo cgqa_corr_hourly


estpost  corr  pm25 avg_pm25_cf1  jaffe_new local_daily satellite_daily
eststo sat_corr_hourly


*********************
*Daily*
******************
use "$final_data_dir/correlations", clear
 
sort datetime
gen pm25_reliable_µgm_full = 0
replace pm25_reliable_µgm_full = 1 if pm25_reliable_µgm !=.
bysort day month year: egen sum_reliable_week = sum(pm25_reliable_µgm_full)
// drop if sum_reliable_week < 24

collapse (mean) pm25_reliable_µgm pm25 avg_pm25_cf1  jaffe_new local_daily  satellite_daily satellite_hourly, by(day month year)


corr pm25_reliable_µgm avg_pm25_cf1
local r = r(rho)




estpost  corr  pm25_reliable_µgm pm25 avg_pm25_cf1  jaffe_new local_daily  satellite_daily
eststo cgqa_corr_daily

estpost  corr  pm25 avg_pm25_cf1  jaffe_new local_daily  satellite_daily
eststo sat_corr_daily


*********************
*Weekly*
******************
gen datetime = mdy(month, day, year)
format datetime %td

* Extract week number from the datetime variable
gen week = week(datetime)

*If uncommented, below only allows weeks with full CGQA data 
sort datetime
gen pm25_reliable_µgm_full = 0
replace pm25_reliable_µgm_full = 1 if pm25_reliable_µgm !=.
bysort week year: egen sum_reliable_week = sum(pm25_reliable_µgm_full)
// drop if sum_reliable_week < 4


collapse (mean) pm25_reliable_µgm pm25 avg_pm25_cf1  jaffe_new local_daily  satellite_daily satellite_hourly, by(week year)


corr pm25_reliable_µgm avg_pm25_cf1
local r = r(rho)

// scatter  pm25_reliable_µgm  avg_pm25_cf1|| lfit  pm25_reliable_µgm  avg_pm25_cf1
// graph save "C:\Users\wb606178\Downloads\weekly.gph", replace

label variable avg_pm25_cf1 "Purple Air"
label variable jaffe_new "Jaffe Correction"
label variable local_daily "Local Calibration"
label variable pm25_reliable_µgm "`Regulatory'"
label variable satellite_daily "Satellite Calibration"
label variable satellite_hourly "Satellite Calibration"
label variable pm25 "Satellite Measures"



estpost  corr  pm25_reliable_µgm pm25 avg_pm25_cf1  jaffe_new local_daily  satellite_daily
eststo cgqa_corr_weekly 


estpost  corr  pm25 avg_pm25_cf1  jaffe_new local_daily  satellite_daily
eststo sat_corr_weekly
matrix Corr = e(b)


 
esttab cgqa_corr_hourly cgqa_corr_daily  cgqa_corr_weekly sat_corr_hourly  sat_corr_daily  sat_corr_weekly using "$outputs/correlation.tex", compress replace label nonumbers nostar type  noobs nonotes booktabs  ///
mtitles("Hourly" "\shortstack{\underline{Regulatory Monitors}\\Daily}" "Weekly" "Hourly" "\shortstack{\underline{Satellite Measures}\\Daily}" "Weekly") ///
 title("Correlations between PM2.5 measures \label{correlation}") ///
     addnotes( "The date range used is for the full year 2023" "The cf=1 formula is used for Purple Air measurements"  "Jaffe correction uses PM1/10 ratio to identify dust events"  "The local correction is the cf=1 Purple Air values calibrated to the local context" "The satellite measure comes from MERRA-2 city-wide averages for Dakar" "The regulatory monitor values come from the average of CGQA regulatory-grade monitors") ///
	 sfmt(%9.2f) b(%9.2f) se(%9.2f) 
	 
	 
******************************************
* END OF SCRIPT
******************************************	 
	 
	 
	 
