*************
*Root Mean Square Error
*************




*clear all
use  "$final_data_dir/correlations", clear

*******************
program define calc_rmse
    syntax varlist(numeric)
    
    
    foreach var of local varlist {
        gen sq_error_`var' = (pm25_reliable - `var')^2
    }

    foreach var of local varlist {
        sum sq_error_`var'
        scalar mse_`var' = r(mean)
        scalar rmse_`var'_full = sqrt(mse_`var')
        scalar rmse_`var' = round(rmse_`var'_full, 0.1)
        di "RMSE for `var' vs regulatory: " rmse_`var'
    }
end

program define calc_rmse_sat
    syntax varlist(numeric)
    
    
    foreach var of local varlist {
        gen sq_error_`var'_sat = (pm25 - `var')^2
    }

    foreach var of local varlist {
        sum sq_error_`var'_sat
        scalar mse_`var'_sat= r(mean)
        scalar rmse_`var'_full_sat = sqrt(mse_`var'_sat)
        scalar rmse_`var'_sat = round(rmse_`var'_full_sat, 0.1)
        di "RMSE for `var' vs regulatory: " rmse_`var'_sat
// 		di "rmse_"`var'"_sat"
    }
end


calc_rmse  pm25 avg_pm25_cf1 jaffe_new local_hourly satellite_hourly

calc_rmse_sat  pm25_reliable avg_pm25_cf1 jaffe_new local_hourly satellite_hourly

***************
*pm25_reliable pm25 avg_pm25_cf1 jaffe_new local_daily satellite_daily
// Define the scalars
*rmse pm25_reliable pm25
scalar hourly1 = rmse_pm25
scalar hourly2 = rmse_avg_pm25_cf1
scalar hourly3 = rmse_jaffe_new
scalar hourly4 = rmse_local_hourly
scalar hourly5 = rmse_satellite_hourly

// scalar hourly6 = rmse_pm25_reliable_µgm_sat
scalar hourly6 = rmse_avg_pm25_cf1_sat
scalar hourly7 = rmse_jaffe_new_sat
scalar hourly8 = rmse_local_hourly_sat
scalar hourly9 = rmse_satellite_hourly_sat

rmse pm25_reliable pm25
rmse pm25_reliable jaffe_new
rmse pm25_reliable satellite_hourly

rmse pm25 avg_pm25_cf1
rmse pm25 local_hourly


*********************
*Daily*
******************
use "$final_data_dir/correlations", clear

collapse (mean) pm25_reliable_µgm pm25 avg_pm25_cf1  jaffe_new local_daily  satellite_daily satellite_hourly, by(day month year)

calc_rmse  pm25 avg_pm25_cf1 jaffe_new local_daily satellite_daily

calc_rmse_sat  pm25_reliable avg_pm25_cf1 jaffe_new local_daily satellite_daily

***************
*pm25_reliable pm25 avg_pm25_cf1 jaffe_new local_daily satellite_daily
// Define the scalars
*rmse pm25_reliable pm25
scalar daily1 = rmse_pm25
scalar daily2 = rmse_avg_pm25_cf1
scalar daily3 = rmse_jaffe_new
scalar daily4 = rmse_local_daily
scalar daily5 = rmse_satellite_daily

// scalar hourly6 = rmse_pm25_reliable_µgm_sat
scalar daily6 = rmse_avg_pm25_cf1_sat
scalar daily7 = rmse_jaffe_new_sat
scalar daily8 = rmse_local_daily_sat
scalar daily9 = rmse_satellite_daily_sat


rmse pm25_reliable pm25
rmse pm25_reliable jaffe_new
rmse pm25_reliable satellite_daily

rmse pm25 avg_pm25_cf1
rmse pm25 local_daily



*********************
*Weekly*
******************
use "$final_data_dir/correlations", clear

collapse (mean) pm25_reliable_µgm pm25 avg_pm25_cf1  jaffe_new local_daily  satellite_daily satellite_hourly, by(week year)

calc_rmse  pm25 avg_pm25_cf1 jaffe_new local_daily satellite_daily

calc_rmse_sat  pm25_reliable avg_pm25_cf1 jaffe_new local_daily satellite_daily

***************
*pm25_reliable pm25 avg_pm25_cf1 jaffe_new local_daily satellite_daily
// Define the scalars
*rmse pm25_reliable pm25
scalar weekly1 = rmse_pm25
scalar weekly2 = rmse_avg_pm25_cf1
scalar weekly3 = rmse_jaffe_new
scalar weekly4 = rmse_local_daily
scalar weekly5 = rmse_satellite_daily

// scalar hourly6 = rmse_pm25_reliable_µgm_sat
scalar weekly6 = rmse_avg_pm25_cf1_sat
scalar weekly7 = rmse_jaffe_new_sat
scalar weekly8 = rmse_local_daily_sat
scalar weekly9 = rmse_satellite_daily_sat

rmse pm25_reliable pm25
rmse pm25_reliable jaffe_new
rmse pm25_reliable satellite_daily

rmse pm25 avg_pm25_cf1
rmse pm25 local_daily

*************
*Generate Matrix 
*************

// Create an empty matrix
matrix rmse = J(5, 6, .)

// Fill the matrix with the defined scalars
matrix rmse[1, 1] = hourly1
matrix rmse[1, 2] = daily1
matrix rmse[1, 3] = weekly1
matrix rmse[2, 1] = hourly2
matrix rmse[2, 2] = daily2
matrix rmse[2, 3] = weekly2
matrix rmse[2, 4] = hourly6
matrix rmse[2, 5] = daily6
matrix rmse[2, 6] = weekly6
matrix rmse[3, 1] = hourly3
matrix rmse[3, 2] = daily3
matrix rmse[3, 3] = weekly3
matrix rmse[3, 4] = hourly7
matrix rmse[3, 5] = daily7
matrix rmse[3, 6] = weekly7
matrix rmse[4, 1] = hourly4
matrix rmse[4, 2] = daily4
matrix rmse[4, 3] = weekly4
matrix rmse[4, 4] = hourly8
matrix rmse[4, 5] = daily8
matrix rmse[4, 6] = weekly8
matrix rmse[5, 1] = hourly5
matrix rmse[5, 2] = daily5
matrix rmse[5, 3] = weekly5
matrix rmse[5, 4] = hourly9
matrix rmse[5, 5] = daily9
matrix rmse[5, 6] = weekly9

// Label the rows and columns of the matrix
matrix rownames rmse = "Satellite Measures" "Purple Air" "Jaffe Correction" "Local Calibration" "Satellite Calibration"
// matrix colnames rmse = "Hourly" "Daily" "Weekly" "Hourly" "Daily" "Weekly"
matrix colnames rmse = "Hourly" "Daily" "Weekly" "Hourly" "Daily" "Weekly"


// Export the matrix to a LaTeX table using esttab
esttab matrix(rmse) using "$outputs/rmse.tex", ///	
mtitles("Regulatory Monitors              Satellite Measures") ///
 title("Root Mean Square Error between PM2.5 measures \label{rmse}") ///
     addnotes( "The date range used is for the full year 2023" "The cf=1 formula is used for Purple Air measurements"  "Jaffe correction uses PM1/10 ratio to identify dust events"  "The satellite measure comes from MERRA-2 city-wide averages for Dakar" "The regulatory monitor values come from the average of CGQA regulatory-grade monitors") ///
   sfmt(%9.2f) b(%9.2f) se(%9.2f) replace noeqlines gaps 
	 


******************************************
* END OF SCRIPT
******************************************	































